#!/bin/zsh

# We're going to run this in the MacTeX working directory...
WORK=`pwd`

# This script fixes the distribution in ``root'' so it can be notarized by Apple. 

# First it makes sure that a small number of binaries in the tlmgr section of TL are signed.
# Since these binaries were submitted to TL by us, they are probably already signed, but
# it is good to make sure.

cd ${WORK}/root/usr/local/texlive/2023/tlpkg/installer

cd lz4
codesign -s   "Developer ID Application: Richard Koch" --force --timestamp --options=runtime lz4.universal-darwin
cd ..

cd xz
codesign -s   "Developer ID Application: Richard Koch" --force --timestamp --options=runtime xz.universal-darwin
cd ..

# The extension pkg indicates a Macintosh install package. Notarization rejects
# any file with this extension. In the past, one or two projects
# used the extension for mere text files. Currently only ctan-o-mat does that.
# In this special case, other files in the project refer to ctan-o-mat.pkg, so changing
# the extension breaks things.

# To fix the problem, we change ctan-o-mat.pkg to ctan-o-mat.txt and add a small
# READ-ME file telling the user to change it back if they use that documentation.
# It is extremely unlikely that anyone uses it.

cd ${WORK}/root/usr/local/texlive/2023/texmf-dist/doc/support/ctan-o-mat
mv ctan-o-mat.pkg ctan-o-mat.txt

cp ${WORK}/README-MACTEX.txt README-MACTEX.txt

# Java projects in TeX Live are stored in the scripts folder. These projects often contain jar files
# contributed by the author. Usually a jar file contains class files, which are interpreted by the
# Java system on the computer; such a system does not come preinstalled on the Mac, but
# can be obtained from Oracle and other sources.

# In rare situations, a jar file can contain precompiled binaries. Apple sees these binaries during
# notarization, and like all other binaries, they must be signed by an Apple developer. Since the
# binaries were not created by the TL Build script, they are not automatically signed by us.
# Note that Apple only inspects darwin binaries for the Mac, so jar files with binaries for other
# platforms will pass notarisation by Apple.

# In 2023, a jar file named albatross was contributed which contained an unsigned darwin binary.
# This file was rejected by Apple. The file had been created by The Island of TeX, and the 
# developers were very helpful. We eventually fixed the problem by removing the darwin binary
# from the jar file. Tests show that it still ran fine; it is possible that a full set of binaries was contributed,
# but only the Windows binaries were used. With the developers permission, we replace the original
# albatross.jar file by our modified file.

cd ${WORK}/root/usr/local/texlive/2023/texmf-dist/scripts/albatross
rm albatross.jar
cp ${WORK}/albatross.jar albatross.jar

# End fixes

# For the record, we explain the exact process used to modify the jar file. We first installed Java.
# The location https://wolfpaulus.com/installing-java-on-macos/ contains a clean modern Mac install
# file. 
#
# It turns out that a jar file is essentially a zip file, so it can be decompressed. When the pieces
# are recompressed to get a jar file back, the process must be given a "manifest" telling the system
# which class file to run first. Java has procedures to uncompress and compress. 
#
#    jar -xf albatross.jar
#
# uncompresses, leaving albatross.jar alone and adding to the folder containing it the following
# six folders and one file: com, kotlin, kotlinx, META-INF, net, org and the file named 
# DebugProbesKt.bin.
#
# The folder com/sun/jna/ contains a folder named darwin. Throw this folder away.
#
# The folder META-INF contains many files and folders, including a file named MANIFEST.MF
# I created a copy of this file at the top level where all these folders live
#
# I renamed albatross.jar to albatross-original.jar so it would not interfere with the last step.
# Finally, the following command recombined all this into a new jar file named albatross.jar:
#
#    jar -cvfm albatross.jar MANIFEST.MF com DebugProbesKt.bin kotlin kotlinx META-INF net org
#
